/**
 * Name: CollisionRectangle.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __COLLISIONRECTANGLE_H__
#define __COLLISIONRECTANGLE_H__

#ifdef __APPLE__
    #include <GraphicsLib/RealRect.h>
    #include <GraphicsLib/Point3.h>
    #include <GraphicsLib/types.h>
    #include <GraphicsLib/Vector.h>
    #include <GraphicsLib/Util.h>
#else
    #include "RealRect.h"
    #include "Point3.h"
    #include "types.h"
    #include "Vector.h"
    #include "Util.h"
#endif

#include "Particle.h"
#include "CollisionObject.h"
#include "Debugger.h"



namespace ParticleSystems {
    //! The CollisionRectangle Class
    /**
    * A class to create a plane which can be used to make Particles collide against.
	* This Class extends the CollisionObject
	* @author Hannes Ricklefs
    */
	class CollisionRectangle : public CollisionObject {
        public:
			//! The Constructor
			/**
			* Constructs a new CollisionRectangle by passing the four corner points. Need to be in clockwise order!
			* \param c1 a pointer to a corner of the CollisionRectangle
			* \param c2 a pointer to a corner of the CollisionRectangle
			* \param c3 a pointer to a corner of the CollisionRectangle
			* \param c4 a pointer to a corner of the CollisionRectangle
			*/
            CollisionRectangle( GraphicsLib::Point3 *c1, GraphicsLib::Point3 *c2, GraphicsLib::Point3 *c3, GraphicsLib::Point3 *c4);
			
			//! The destructor
			/**
			* The Destructor of this CollisionRectangle
			*/
			virtual ~CollisionRectangle();
			
            //! A virtual function
			/**
			* Checks if a collision occured between the passed Particle
			* and this CollisionRectangle
			* \param p a pointer to the Particle
			*/
		    virtual void collide(Particle *p);
			
			//! A virtual function
			/**
			* Draws this CollisionRectangle
			*/
            virtual void draw();
			
			//! A virtual function
			/**
			* Translates this CollisionRectangle
			*/
            virtual void translate();
			
			//! A virtual function
			/**
			* Translates this CollisionRectangle
			*/
            virtual void rotate();
			
			//! A virtual function
			/**
			* Translates this CollisionSphere
			*/
            virtual void scale();
			
			//! A member function
			/**
			* Returns the distance between the given vector and the point
			* \param v the pointer to the vector
			* \param p the pointer to the point
			*/
            GraphicsLib::Real getDistance(GraphicsLib::Vector *v, GraphicsLib::Point3 *p);
			
			//! A member function
			/**
			* Calculates the center of this Rectangle and stores the result
			* in the center point.
			*/
            void calcCenter();
			
			//! A member function
			/**
			* Calculates the normal of this CollisionRectangle and stores the result
			* in the normal vector.
			*/
            void calculateNormal();
			
			//! A member function
			/**
			* Calculate the height of the bounding box for this Rectangle
			*/
			GraphicsLib::Real BBheight();
			
			//! A member function
			/**
			* Calculate the width of the bounding box for this Rectangle
			*/
			GraphicsLib::Real BBwidth();
			
			//! A member function
			/**
			* Calculate the depth of the bounding box for this Rectangle
			*/
			GraphicsLib::Real BBdepth();
			
			//! A member Function
			/**
			* The maximum of two Real numbers
			* \param a the first test
			* \param b the second test
			* \return the max of the two
			*/
			float max(float a, float b);
			
			//! A member Function
			/**
			* The minimum of two Real numbers
			* \param a the first test
			* \param b the second test
			* \return the min of the two
			*/
			float min(float a, float b);
			
			//! A public variable
			/**
			* Corner1 of the CollisionRectangle
			*/
            GraphicsLib::Point3 corner1;
            
			//! A public variable
			/**
			* Corner2 of the CollisionRectangle
			*/
			GraphicsLib::Point3 corner2;
            
			//! A public variable
			/**
			* Corner3 of the CollisionRectangle
			*/
			GraphicsLib::Point3 corner3;
            
			//! A public variable
			/**
			* Corner4 of the CollisionRectangle
			*/
			GraphicsLib::Point3 corner4;
			
			//! A public variable
			/**
			* The Center of the CollisionRectangle
			*/
            GraphicsLib::Point3 center;
			
			//! A public variable
			/**
			*The normal of the CollisionRectangle
			*/
            GraphicsLib::Vector normal;

	};
}
#endif

