/**
 * Name: DataStructure.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __DATASTRUCTURE_H__
#define __DATASTRUCTURE_H__

#ifdef __APPLE__
	#include <GraphicsLib/Point3.h>
#else
	#include "Point3.h"
#endif

#include "Particle.h"
#include "CollisionObject.h"

namespace ParticleSystems {
	//! The DataStructure Class
	/**
	* The DataStructure Class is the Superclass for all Classes that want to 
	* store the Particles for a ParticleSystem. Its main responsibility is to
	* store Particles and the management of these Particles. A DataStructure should 
	* store new Particles and store dead Particles for rebirth. In addition it should
	* store any Objects in the scene with which Particles can collide. 
	* @author Hannes Ricklefs
	*/
	class DataStructure {
	public:
		
		//! The Constructor
		/**
		* Constructs a new DataStructure
		*/
		DataStructure();
		
		//! The Destructor
		/**
		* Destructs a DataStructure
		*/
		virtual ~DataStructure();
		
		//! An abstract Function
		/**
		* Adds a new Particle.
		* \param particle a pointer to the new Particle
		*/
		virtual void addParticles(Particle *particle) = 0;
		
		//! An abstract Function
		/**
		* Removes all Particles
		*/
		virtual void killParticles() = 0;
        
		//! An abstract Function
		/**
		* Add a new CollisionObject to the DataStructure
		* \param co a pointer to the CollisionObject
		*/
		virtual void addCollisionObject(CollisionObject *co) = 0;
        
		//! An abstract Function
		/**
		* Deletes the given CollisionObject
		* \param co a pointer to the about to be deleted CollisionObject
		*/
		virtual void deleteCollisionObject(CollisionObject *co) = 0;
		
		//! An abstract Function
		/**
		* Checks is the given point is colliding with any of the 
		* CollisionObjects.
		* \param pos the position to be checked
		* \return True if in collision range
		*/
		virtual bool positionCheck(GraphicsLib::Point3 pos) = 0;
		
		//! An abstract Function
		/**
		* Evolves the Particles within the DataStructure
		*/
		virtual void evolve() = 0;
        
		//! An abstract Function
		/**
		* Draws all the Particle and CollisionObjects within this
		* DataStructure
		*/
		virtual void draw() = 0;
		
		//! An abstract Function
		/** 
		* Return the number of Particles within the DataStructure
		* as an int.
		* \return The number of Particles
		*/
		virtual int numParticles() = 0;
		
		//! An abstract Function
		/**
		* Returns true if the DataStructure contains any
		* dead Particles
		* \return True is dead Particles are stored
		*/
		virtual bool hasDead() = 0;
		
		//! An abstract Function
		/**
		* Returns a pointer to a dead Particle
		* !! Should only be used if hasDead returns True !!
		* Otherwise NULL will be returned.
		* \return A pointer to a dead Particle.
		*/
		virtual Particle* getDead() = 0;
		
		//! A public variable
		/**
		* Specifies how many need to be stored 
		* inorder not to save too many dead particles
		*/
		int storageCapacity;
        
		//! A public variable
		/**
		* If set to true collision tests will be performed
		* set to false as default
		*/
		bool collision;
	};
}
#endif

		
