/**
 * Name: Emitter.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __EMITTER_H__
#define __EMITTER_H__

#ifdef __APPLE__
	#include <GraphicsLib/Point3.h>
	#include <GraphicsLib/Vector.h>
#else
	#include "Point3.h"
	#include "Vector.h"
#endif

#include "Particle.h"
#include "Solver.h"
#include "Force.h"
#include "Texture.h"
#include "TGALoader.h"

namespace ParticleSystems {
	//! The Emitter Class
	/**
	* The Emitter Class is the Superclass for all Emitters. The functionality of a Emitter
	* is to initialise new Particles or reBirth dead Particles. 
	* @author Hannes Ricklefs
	*/
	class Emitter {
		
	public:
		
		//! A public enum
		/**
		* Defines the possible types a Particle can be namely Point, Image, and Strike
		*/ 
		enum PARTICLETYPE {POINT, IMAGE, STRIKE};
		
		//! The Constructor
		/**
		* Constructs a new Emitter
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		*/
		Emitter(int SYSID, 
				Solver *s, 
				PARTICLETYPE type, 
				GraphicsLib::Vector initVel, 
				GraphicsLib::Colour initialColor, 
				GLfloat pSize); 
				
		//! The Constructor
		/**
		* Constructs a new Emitter with a Texture
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		* \param TexName a pointer to a texture
		*/
		Emitter(int SYSID, 
				Solver *s, 
				PARTICLETYPE type, 
				GraphicsLib::Vector initVel, 
				GraphicsLib::Colour initialColor, 
				GLfloat pSize, 
				char *TexName); 
		
		//! The Destructor
		/**
		* Destructs this Emitter
		*/
		virtual ~Emitter();
		
		//! An abstract Function
		/**
		* Emitts a new Particle according to the values
		* \return A pointer to the new Particle
		*/
		virtual Particle* emittParticles(void) = 0;
		
		//! An abstract Function
		/**
		* Reinitialises the given Particle as it would be newly created
		* \param P a pointer to the Particle to be reborne
		*/
		virtual void reBirth(Particle *p) = 0;
		
		//! A public variable
		/**
		* Stores the Particle type of this emitter
		*/
		PARTICLETYPE type;
        
		//! A public variable
		/**
		* Stores the texture for the ImgParticle
		*/
		Texture texture;
		
		//! A public variable
		/**
		* The particleLife
		*/
		GLfloat particleLife;
		
		//! A public variable
		/**
		* The Particle size
		*/
		GLfloat particleSize;
		
		//! A public variable
		/**
		* The Particle flare size
		*/
		GLfloat particleSizeFlare;
		
		//! A public Variable
		/**
		* Inital velocity of the particle to be emitted
		*/
		GraphicsLib::Vector initialVelocity;
		
		//! A public Variable
		/**
		* Inital colour of the particle to be emitted
		*/
		GraphicsLib::Colour initialColor;
		
		//! A public Variable
		/**
		* The flare Colour of the particle to be emitted
		*/
		GraphicsLib::Colour flareColor;
		
		//! A public Variable
		/**
		* The Solver for this Emitter
		*/
		Solver *solver;
		
	private:
		//! A private Function
		/**
		* Initialises the Emitter. Sets some default values. No flare for Size and Colour.
		*/
		void initEmitter();
		
	protected:
		//! A protected Variable
		/**
		* The current ID of the ParticleSystem this Emitter belongs to
		*/
		int SYSID;
		
		//! A protected Variable
		/**
		* The TGALoader for ImgParticles
		*/
		TGALoader *loader;
	};
}
#endif

